/*
    FreeRTOS V6.1.1
 *
 */

/* Standard includes. */
#include <stdio.h>

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"

/* Library includes. */
#include "stm32f10x_it.h"

/* Demo app includes. */
#include "hardware.h"

extern uint16_t ADCConvertedValue[2][BUF_SIZE];

xSemaphoreHandle xSemaphore = NULL;

static void vLEDTask(void *pvParameters);
static void vADCTask(void *pvParameters);

int main(void)
{
#ifdef DEBUG
  debug();
#endif

  Hardware_Init();

  vSemaphoreCreateBinary(xSemaphore);

  xTaskCreate(vLEDTask, (signed portCHAR *) "LED", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY + 1, NULL);
  xTaskCreate(vADCTask, (signed portCHAR *) "ADC", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY + 3, NULL);

  /* Start the scheduler. */
  vTaskStartScheduler();
  /* Will only get here if there was not enough heap space to create the
     idle task. */
//  for (;;);
  return 0;
}

uint16_t isqrt32(uint32_t x)
{
  uint32_t m, y, b;
  m = 0x40000000;
  y = 0;
  while (m != 0) {
    b = y | m;
    y = y >> 1;
    if (x >= b) {
      x = x - b;
      y = y | m;
    }
    m >>= 2;
  }
  return y;
}

uint16_t rms(uint16_t * ptr)
{
  uint64_t t=0;
  uint16_t i;

  for (i = 0; i < BUF_SIZE; i++)
    t += (uint32_t) * ptr * (uint32_t) * ptr;
  return (isqrt32(t / BUF_SIZE));
}

void vADCTask(void *pvParameters)
{
//  const portTickType xDelay = 500 / portTICK_RATE_MS;

  __IO uint8_t ii=0;
  vTaskDelay((portTickType) 100);
  for (;;) {
    GPIO_SetBits(GPIOB, GPIO_Pin_14);  // for debug only. @#$
    if (xSemaphoreTake(xSemaphore, (portTickType) 1) == pdTRUE) {
      DMA_ADC_Configuration(ii);
      PWM_Output(ii, rms(&ADCConvertedValue[ii][0])/4);
      GPIO_ResetBits(GPIOB, GPIO_Pin_14); // for debug only. @#$
    } else {
      // Invoke Watchdog
    }
    if(++ii>1)ii=0;

  }
}

void vLEDTask(void *pvParameters)
{
  const portTickType xDelay = 500 / portTICK_RATE_MS;

  for (;;) {
    /* Simply toggle the LED every 500ms, blocking between each toggle. */
    vToggleLED();
    vTaskDelay(xDelay);
  }
}

/*-----------------------------------------------------------*/

#ifdef  DEBUG
/* Keep the linker happy. */
void assert_failed(unsigned portCHAR * pcFile, unsigned portLONG ulLine)
{
  for (;;) {
  }
}
#endif
